# frozen_string_literal: true

#
# The authentication part of Application Controller
#
module Authentication
  module Authenticate
    extend ActiveSupport::Concern

    included do
      include Pundit

      rescue_from Pundit::NotAuthorizedError, with: :user_not_authorized

      before_action :configure_permitted_parameters, if: :devise_controller?
      after_action :verify_authorized, unless: :devise_controller?
      after_action :verify_policy_scoped, unless: :devise_controller?
      before_action :configure_permitted_parameters, if: :devise_controller?
      before_action :store_current_location, unless: :devise_controller?

      def pundit_user
        Current
      end

      private

      def user_not_authorized
        alert_msg = t('you_do_not_have_permission_to_access_this_page')
        redirect_to main_app.root_path, alert: alert_msg
      end

      # override the devise helper to store the current location so we can
      # redirect to it after loggin in or out. This override makes signing in
      # and signing up work automatically.
      def store_current_location
        store_location_for(:user, request.url)
      end

      protected

      def configure_permitted_parameters
        devise_parameter_sanitizer.permit(
          :account_update,
          keys: %i[username email time_zone password password_confirmation
                   current_password]
        )
        devise_parameter_sanitizer.permit(
          :sign_up,
          keys: %i[username email time_zone password password_confirmation
                   current_password]
        )
      end

      def configure_permitted_parameters
        added_attrs = [:username, :email, :password, :password_confirmation, :remember_me, :timezone, :language, :username, :firstname, :secondname, :thirdname]
        devise_parameter_sanitizer.permit :sign_up, keys: added_attrs
        devise_parameter_sanitizer.permit :account_update, keys: added_attrs
      end
    end
  end
end