module Authentication
  class User < ApplicationRecord
    # Include default devise modules. Others available are:
    #  and :omniauthable
    devise :database_authenticatable, :registerable,
           :recoverable, :rememberable, :trackable, :validatable,
           :confirmable, :lockable, :timeoutable
    
    rolify role_join_table_name: :authentication_users_authentication_roles, role_cname: 'Authentication::Role'

    has_and_belongs_to_many :companies, optional: true
    belongs_to :current_company, foreign_key: :current_company_id, class_name: 'Authentication::Company', optional: true
    before_create :set_locale
    after_create :create_company
    after_create :assign_default_role
    after_create :check_invitations

    validate :validate_username
    validates :username, presence: :true, uniqueness: { case_sensitive: false }
    validates_format_of :username, with: /^[a-zA-Z0-9_\.]*$/, multiline: true

    attr_accessor :login

    def validate_username
      if User.where(email: username).exists?
        errors.add(:username, :invalid)
      end
    end

    def login=(login)
      @login = login
    end

    def login
      @login || self.username || self.email
    end

    def current_company?
      if current_company && Authentication::Company.exists?(current_company_id)
        return current_company
      else
        self.current_company = self.companies.where(user_id: self.id).try(:first)
        unless self.current_company
          self.current_company = self.companies.order(:id).try(:first)
          self.save(validation: false)
        end
        # self.reload
        return self.current_company
      end
    end

    def company?
      companies.count > 0
    end

    def to_s
      name = [self.firstname, self.secondname, self.thirdname].join(' ')
      if name.blank?
        self.username.blank? ? self.email : self.username
      else
        name
      end
    end

    def create_company
      my_company = Authentication::Company.create!(name: 'Pénzügyeim', user: self)
      my_company.user = self
      my_company.save!
      self.companies << my_company
      self.add_role(:member, my_company) unless self.has_role?(:member, my_company)
    end

    def assign_default_role
      self.add_role(:member) unless self.has_role?(:member)
    end

    def check_invitations
      invitations = Authentication::Invitation.where(email: self.email, used: false)
      invitations.each do |invitation|
        company = Authentication::Company.find(invitation.company_id)
        Company.rights.each do |right|
          self.add_role(right, company) unless self.has_role?(right, company)
          self.save!
        end

        unless company.users.include? self
          company.users << self
          company.save!
        end
        
        conn = ActiveRecord::Base.connection
        conn.execute(%Q{
            update authentication_invitations set used=true where email=#{conn.quote(invitation.email)} and user_id=#{conn.quote(invitation.user_id)} and company_id=#{conn.quote(invitation.company_id)}
        })
      end
    end

    protected
    
    def self.find_for_database_authentication(warden_conditions)
      conditions = warden_conditions.dup
      puts "-----"
      puts conditions.inspect
      if login = conditions.delete(:login)
        where(conditions.to_hash).where(["lower(username) = :value OR lower(email) = :value", { value: login.downcase }]).first
      elsif conditions.has_key?(:username) || conditions.has_key?(:email)
        conditions[:email].downcase! if conditions[:email]
        where(conditions.to_hash).first
      end
    end

    private

    def set_locale
      locale = I18n.locale  
    end
  end
end
